@@ -1836,7 +1836,6 @@ int old_snapshot_add(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
   header = (struct rbd_obj_header_ondisk *)bl.c_str();
 
   int snaps_id_ofs = sizeof(*header);
-  int len = snaps_id_ofs;
   int names_ofs = snaps_id_ofs + sizeof(*new_snaps) * header->snap_count;
   const char *snap_name;
   const char *snap_names = ((char *)header) + names_ofs;
@@ -1885,8 +1884,6 @@ int old_snapshot_add(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
   new_snaps[0].id = snap_id;
   new_snaps[0].image_size = header->image_size;
 
-  len += sizeof(*new_snaps) * header->snap_count + header->snap_names_len;
-
   memcpy(header_bp.c_str(), header, sizeof(*header));
 
   newbl.push_back(header_bp);
@@ -1906,7 +1903,6 @@ int old_snapshot_remove(cls_method_context_t hctx, bufferlist *in, bufferlist *o
   struct rbd_obj_header_ondisk *header;
   bufferlist newbl;
   bufferptr header_bp(sizeof(*header));
-  struct rbd_obj_snap_ondisk *new_snaps;
 
   int rc = snap_read_header(hctx, bl);
   if (rc < 0)
@@ -1915,7 +1911,7 @@ int old_snapshot_remove(cls_method_context_t hctx, bufferlist *in, bufferlist *o
   header = (struct rbd_obj_header_ondisk *)bl.c_str();
 
   int snaps_id_ofs = sizeof(*header);
-  int names_ofs = snaps_id_ofs + sizeof(*new_snaps) * header->snap_count;
+  int names_ofs = snaps_id_ofs + sizeof(struct rbd_obj_snap_ondisk) * header->snap_count;
   const char *snap_name;
   const char *snap_names = ((char *)header) + names_ofs;
   const char *orig_names = snap_names;
@@ -1982,7 +1978,6 @@ int old_snapshot_remove(cls_method_context_t hctx, bufferlist *in, bufferlist *o
     return rc;
 
   return 0;
-
 }
 
 
