@@ -20,6 +20,25 @@ void CrushWrapper::find_roots(set<int>& roots) const
   }
 }
 
+bool CrushWrapper::subtree_contains(int root, int item) const
+{
+  if (root == item)
+    return true;
+
+  if (root >= 0)
+    return false;  // root is a leaf
+
+  const crush_bucket *b = get_bucket(root);
+  if (!b)
+    return false;
+
+  for (unsigned j=0; j<b->size; j++) {
+    if (subtree_contains(b->items[j], item))
+      return true;
+  }
+  return false;
+}
+
 
 int CrushWrapper::remove_item(CephContext *cct, int item)
 {
@@ -183,6 +202,23 @@ map<int, string> CrushWrapper::get_parent_hierarchy(int id)
   return parent_hierarchy;
 }
 
+int CrushWrapper::get_children(int id, list<int> *children)
+{
+  // leaf?
+  if (id >= 0) {
+    return 0;
+  }
+
+  crush_bucket *b = get_bucket(id);
+  if (!b) {
+    return -ENOENT;
+  }
+
+  for (unsigned n=0; n<b->size; n++) {
+    children->push_back(b->items[n]);
+  }
+  return b->size;
+}
 
 
 int CrushWrapper::insert_item(CephContext *cct, int item, float weight, string name,
@@ -230,9 +266,23 @@ int CrushWrapper::insert_item(CephContext *cct, int item, float weight, string n
       return -EINVAL;
     }
 
+    // check that we aren't creating a cycle.
+    if (subtree_contains(id, cur)) {
+      ldout(cct, 1) << "insert_item item " << cur << " already exists beneath " << id << dendl;
+      return -EINVAL;
+    }
+
     crush_bucket *b = get_bucket(id);
     assert(b);
 
+    if (p->first != b->type) {
+      ldout(cct, 1) << "insert_item existing bucket has type "
+	<< "'" << type_map[b->type] << "' != "
+	<< "'" << type_map[p->first] << "'" << dendl;
+      return -EINVAL;
+    }
+
+
     // make sure the item doesn't already exist in this bucket
     for (unsigned j=0; j<b->size; j++)
       if (b->items[j] == cur) {
@@ -299,7 +349,7 @@ int CrushWrapper::create_or_move_item(CephContext *cct, int item, float weight,
     }
     ldout(cct, 5) << "create_or_move_item adding " << item << " weight " << weight
 		  << " at " << loc << dendl;
-    ret = insert_item(cct, item, weight, name.c_str(), loc);
+    ret = insert_item(cct, item, weight, name, loc);
     if (ret == 0)
       ret = 1;  // changed
   }
@@ -335,7 +385,7 @@ int CrushWrapper::update_item(CephContext *cct, int item, float weight, string n
     }
     ldout(cct, 5) << "update_item adding " << item << " weight " << weight
 		  << " at " << loc << dendl;
-    ret = insert_item(cct, item, weight, name.c_str(), loc);
+    ret = insert_item(cct, item, weight, name, loc);
     if (ret == 0)
       ret = 1;  // changed
   }
@@ -393,24 +443,36 @@ pair<string,string> CrushWrapper::get_immediate_parent(int id)
 {
   pair <string, string> loc;
 
-
   for (int bidx = 0; bidx < crush->max_buckets; bidx++) {
     crush_bucket *b = crush->buckets[bidx];
     if (b == 0)
       continue;
     for (unsigned i = 0; i < b->size; i++)
-      if (b->items[i] == id){
+      if (b->items[i] == id) {
         string parent_id = name_map[b->id];
         string parent_bucket_type = type_map[b->type];
         loc = make_pair(parent_bucket_type, parent_id);
       }
   }
 
-
   return loc;
 }
 
-
+int CrushWrapper::get_immediate_parent_id(int id, int *parent)
+{
+  for (int bidx = 0; bidx < crush->max_buckets; bidx++) {
+    crush_bucket *b = crush->buckets[bidx];
+    if (b == 0)
+      continue;
+    for (unsigned i = 0; i < b->size; i++) {
+      if (b->items[i] == id) {
+	*parent = b->id;
+	return 0;
+      }
+    }
+  }
+  return -ENOENT;
+}
 
 void CrushWrapper::reweight(CephContext *cct)
 {
@@ -426,6 +488,61 @@ void CrushWrapper::reweight(CephContext *cct)
   }
 }
 
+int CrushWrapper::add_simple_rule(string name, string root_name, string failure_domain_name)
+{
+  if (rule_exists(name))
+    return -EEXIST;
+  if (!name_exists(root_name.c_str()))
+    return -ENOENT;
+  int root = get_item_id(root_name.c_str());
+  int type = 0;
+  if (failure_domain_name.length()) {
+    type = get_type_id(failure_domain_name.c_str());
+    if (type <= 0) // bah, returns 0 on error; but its ok, device isn't a domain really
+      return -EINVAL;
+  }
+
+  int ruleset = 0;
+  for (int i = 0; i < get_max_rules(); i++) {
+    if (rule_exists(i) &&
+	get_rule_mask_ruleset(i) >= ruleset) {
+      ruleset = get_rule_mask_ruleset(i) + 1;
+    }
+  }
+
+  crush_rule *rule = crush_make_rule(3, ruleset, 1 /* pg_pool_t::TYPE_REP */, 1, 10);
+  assert(rule);
+  crush_rule_set_step(rule, 0, CRUSH_RULE_TAKE, root, 0);
+  if (type)
+    crush_rule_set_step(rule, 1,
+			CRUSH_RULE_CHOOSE_LEAF_FIRSTN,
+			CRUSH_CHOOSE_N,
+			type);
+  else
+    crush_rule_set_step(rule, 1,
+			CRUSH_RULE_CHOOSE_FIRSTN,
+			CRUSH_CHOOSE_N,
+			0);
+  crush_rule_set_step(rule, 2, CRUSH_RULE_EMIT, 0, 0);
+  int rno = crush_add_rule(crush, rule, -1);
+  set_rule_name(rno, name.c_str());
+  have_rmaps = false;
+  return rno;
+}
+
+int CrushWrapper::remove_rule(int ruleno)
+{
+  if (ruleno >= (int)crush->max_rules)
+    return -ENOENT;
+  if (crush->rules[ruleno] == NULL)
+    return -ENOENT;
+  crush_destroy_rule(crush->rules[ruleno]);
+  crush->rules[ruleno] = NULL;
+  rule_name_map.erase(ruleno);
+  have_rmaps = false;
+  return 0;
+}
+
 void CrushWrapper::encode(bufferlist& bl, bool lean) const
 {
   assert(crush);
@@ -507,6 +624,7 @@ void CrushWrapper::encode(bufferlist& bl, bool lean) const
   ::encode(crush->choose_local_tries, bl);
   ::encode(crush->choose_local_fallback_tries, bl);
   ::encode(crush->choose_total_tries, bl);
+  ::encode(crush->chooseleaf_descend_once, bl);
 }
 
 static void decode_32_or_64_string_map(map<int32_t,string>& m, bufferlist::iterator& blp)
@@ -582,6 +700,9 @@ void CrushWrapper::decode(bufferlist::iterator& blp)
       ::decode(crush->choose_local_fallback_tries, blp);
       ::decode(crush->choose_total_tries, blp);
     }
+    if (!blp.end()) {
+      ::decode(crush->chooseleaf_descend_once, blp);
+    }
     finalize();
   }
   catch (...) {
@@ -751,6 +872,12 @@ void CrushWrapper::dump(Formatter *f) const
   f->close_section();
 
   f->open_array_section("rules");
+  dump_rules(f);
+  f->close_section();
+}
+
+void CrushWrapper::dump_rules(Formatter *f) const
+{
   for (int i=0; i<get_max_rules(); i++) {
     if (!rule_exists(i))
       continue;
@@ -806,7 +933,15 @@ void CrushWrapper::dump(Formatter *f) const
     f->close_section();
     f->close_section();
   }
-  f->close_section();
+}
+
+void CrushWrapper::list_rules(Formatter *f) const
+{
+  for (int rule = 0; rule < get_max_rules(); rule++) {
+    if (!rule_exists(rule))
+      continue;
+    f->dump_string("name", get_rule_name(rule));
+  }
 }
 
 void CrushWrapper::generate_test_instances(list<CrushWrapper*>& o)
