@@ -34,6 +34,8 @@
 #define CEPH_FEATURE_REPLY_CREATE_INODE   (1<<27)
 #define CEPH_FEATURE_OSD_HBMSGS     (1<<28)
 #define CEPH_FEATURE_MDSENC         (1<<29)
+#define CEPH_FEATURE_OSDHASHPSPOOL  (1<<30)
+#define CEPH_FEATURE_MON_SINGLE_PAXOS (1<<31)
 
 /*
  * Features supported.  Should be everything above.
@@ -67,8 +69,10 @@
 	 CEPH_FEATURE_CRUSH_TUNABLES2 |	     \
 	 CEPH_FEATURE_CREATEPOOLID |	     \
 	 CEPH_FEATURE_REPLY_CREATE_INODE |   \
-	 CEPH_FEATURE_OSD_HBMSGS | \
-	 CEPH_FEATURE_MDSENC)
+	 CEPH_FEATURE_OSD_HBMSGS |		\
+	 CEPH_FEATURE_MDSENC |			\
+	 CEPH_FEATURE_OSDHASHPSPOOL |       \
+	 CEPH_FEATURE_MON_SINGLE_PAXOS)
 
 #define CEPH_FEATURES_SUPPORTED_DEFAULT  CEPH_FEATURES_ALL
 
