@@ -37,6 +37,7 @@ public:
   ceph_mount_info(uint64_t msgr_nonce_, CephContext *cct_)
     : msgr_nonce(msgr_nonce_),
       mounted(false),
+      inited(false),
       client(NULL),
       monclient(NULL),
       messenger(NULL),
@@ -95,6 +96,8 @@ public:
     if (ret)
       goto fail;
 
+    inited = true;
+
     ret = client->mount(mount_root);
     if (ret)
       goto fail;
@@ -121,8 +124,9 @@ public:
       client->unmount();
       mounted = false;
     }
-    if (client) {
+    if (inited) {
       client->shutdown();
+      inited = false;
     }
     if (messenger) {
       messenger->shutdown();
@@ -201,6 +205,7 @@ public:
 private:
   uint64_t msgr_nonce;
   bool mounted;
+  bool inited;
   Client *client;
   MonClient *monclient;
   Messenger *messenger;
@@ -569,21 +574,21 @@ extern "C" int ceph_fchmod(struct ceph_mount_info *cmount, int fd, mode_t mode)
   return cmount->get_client()->fchmod(fd, mode);
 }
 extern "C" int ceph_chown(struct ceph_mount_info *cmount, const char *path,
-			  uid_t uid, gid_t gid)
+			  int uid, int gid)
 {
   if (!cmount->is_mounted())
     return -ENOTCONN;
   return cmount->get_client()->chown(path, uid, gid);
 }
 extern "C" int ceph_fchown(struct ceph_mount_info *cmount, int fd,
-			   uid_t uid, gid_t gid)
+			   int uid, int gid)
 {
   if (!cmount->is_mounted())
     return -ENOTCONN;
   return cmount->get_client()->fchown(fd, uid, gid);
 }
 extern "C" int ceph_lchown(struct ceph_mount_info *cmount, const char *path,
-			   uid_t uid, gid_t gid)
+			   int uid, int gid)
 {
   if (!cmount->is_mounted())
     return -ENOTCONN;
