@@ -70,63 +70,22 @@ void librados::IoCtxImpl::queue_aio_write(AioCompletionImpl *c)
   aio_write_list_lock.Lock();
   assert(c->io == this);
   c->aio_write_seq = ++aio_write_seq;
-  ldout(client->cct, 20) << "queue_aio_write " << this << " completion " << c
-			 << " write_seq " << aio_write_seq << dendl;
   aio_write_list.push_back(&c->aio_write_list_item);
   aio_write_list_lock.Unlock();
 }
 
 void librados::IoCtxImpl::complete_aio_write(AioCompletionImpl *c)
 {
-  ldout(client->cct, 20) << "complete_aio_write " << c << dendl;
   aio_write_list_lock.Lock();
   assert(c->io == this);
   c->aio_write_list_item.remove_myself();
-
-  map<tid_t, std::list<AioCompletionImpl*> >::iterator waiters = aio_write_waiters.begin();
-  while (waiters != aio_write_waiters.end()) {
-    if (!aio_write_list.empty() &&
-	aio_write_list.front()->aio_write_seq <= waiters->first) {
-      ldout(client->cct, 20) << " next outstanding write is " << aio_write_list.front()->aio_write_seq
-			     << " <= waiter " << waiters->first
-			     << ", stopping" << dendl;
-      break;
-    }
-    ldout(client->cct, 20) << " waking waiters on seq " << waiters->first << dendl;
-    for (std::list<AioCompletionImpl*>::iterator it = waiters->second.begin();
-	 it != waiters->second.end(); ++it) {
-      client->finisher.queue(new C_AioCompleteAndSafe(*it));
-      (*it)->put();
-    }
-    aio_write_waiters.erase(waiters++);
-  }
-
   aio_write_cond.Signal();
   aio_write_list_lock.Unlock();
   put();
 }
 
-void librados::IoCtxImpl::flush_aio_writes_async(AioCompletionImpl *c)
-{
-  ldout(client->cct, 20) << "flush_aio_writes_async " << this
-			 << " completion " << c << dendl;
-  Mutex::Locker l(aio_write_list_lock);
-  tid_t seq = aio_write_seq;
-  if (aio_write_list.empty()) {
-    ldout(client->cct, 20) << "flush_aio_writes_async no writes. (tid "
-			   << seq << ")" << dendl;
-    client->finisher.queue(new C_AioCompleteAndSafe(c));
-  } else {
-    ldout(client->cct, 20) << "flush_aio_writes_async " << aio_write_list.size()
-			   << " writes in flight; waiting on tid " << seq << dendl;
-    c->get();
-    aio_write_waiters[seq].push_back(c);
-  }
-}
-
 void librados::IoCtxImpl::flush_aio_writes()
 {
-  ldout(client->cct, 20) << "flush_aio_writes" << dendl;
   aio_write_list_lock.Lock();
   tid_t seq = aio_write_seq;
   while (!aio_write_list.empty() &&
@@ -716,8 +675,7 @@ int librados::IoCtxImpl::aio_operate_read(const object_t &oid,
 }
 
 int librados::IoCtxImpl::aio_operate(const object_t& oid,
-				     ::ObjectOperation *o, AioCompletionImpl *c,
-				     const ::SnapContext& snap_context)
+				     ::ObjectOperation *o, AioCompletionImpl *c)
 {
   utime_t ut = ceph_clock_now(client->cct);
   /* can't write to a snapshot */
@@ -731,15 +689,13 @@ int librados::IoCtxImpl::aio_operate(const object_t& oid,
   queue_aio_write(c);
 
   Mutex::Locker l(*lock);
-  objecter->mutate(oid, oloc, *o, snap_context, ut, 0, onack, oncommit,
-		   &c->objver);
+  objecter->mutate(oid, oloc, *o, snapc, ut, 0, onack, oncommit, &c->objver);
 
   return 0;
 }
 
 int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
-				  bufferlist *pbl, size_t len, uint64_t off,
-				  uint64_t snapid)
+				  bufferlist *pbl, size_t len, uint64_t off)
 {
   if (len > (size_t) INT_MAX)
     return -EDOM;
@@ -753,14 +709,13 @@ int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
 
   Mutex::Locker l(*lock);
   objecter->read(oid, oloc,
-		 off, len, snapid, &c->bl, 0,
+		 off, len, snap_seq, &c->bl, 0,
 		 onack, &c->objver);
   return 0;
 }
 
 int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
-				  char *buf, size_t len, uint64_t off,
-				  uint64_t snapid)
+				  char *buf, size_t len, uint64_t off)
 {
   if (len > (size_t) INT_MAX)
     return -EDOM;
@@ -774,7 +729,7 @@ int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
 
   Mutex::Locker l(*lock);
   objecter->read(oid, oloc,
-		 off, len, snapid, &c->bl, 0,
+		 off, len, snap_seq, &c->bl, 0,
 		 onack, &c->objver);
 
   return 0;
@@ -784,7 +739,7 @@ int librados::IoCtxImpl::aio_sparse_read(const object_t oid,
 					 AioCompletionImpl *c,
 					 std::map<uint64_t,uint64_t> *m,
 					 bufferlist *data_bl, size_t len,
-					 uint64_t off, uint64_t snapid)
+					 uint64_t off)
 {
   if (len > (size_t) INT_MAX)
     return -EDOM;
@@ -797,7 +752,7 @@ int librados::IoCtxImpl::aio_sparse_read(const object_t oid,
 
   Mutex::Locker l(*lock);
   objecter->sparse_read(oid, oloc,
-		 off, len, snapid, &c->bl, 0,
+		 off, len, snap_seq, &c->bl, 0,
 		 onack);
   return 0;
 }
@@ -896,6 +851,21 @@ int librados::IoCtxImpl::aio_remove(const object_t &oid, AioCompletionImpl *c)
   return 0;
 }
 
+
+int librados::IoCtxImpl::aio_stat(const object_t& oid, AioCompletionImpl *c,
+				  uint64_t *psize, time_t *pmtime)
+{
+  c->io = this;
+  C_aio_stat_Ack *onack = new C_aio_stat_Ack(c, pmtime);
+
+  Mutex::Locker l(*lock);
+  objecter->stat(oid, oloc,
+		 snap_seq, psize, &onack->mtime, 0,
+		 onack, &c->objver);
+
+  return 0;
+}
+
 int librados::IoCtxImpl::remove(const object_t& oid)
 {
   utime_t ut = ceph_clock_now(client->cct);
@@ -1433,7 +1403,7 @@ int librados::IoCtxImpl::watch(const object_t& oid, uint64_t ver,
   lock->Lock();
 
   WatchContext *wc = new WatchContext(this, oid, ctx);
-  client->register_watcher(wc, oid, ctx, cookie);
+  client->register_watcher(wc, cookie);
   prepare_assert_ops(&wr);
   wr.watch(*cookie, ver, 1);
   bufferlist bl;
@@ -1461,12 +1431,14 @@ int librados::IoCtxImpl::watch(const object_t& oid, uint64_t ver,
 
 
 /* this is called with IoCtxImpl::lock held */
-int librados::IoCtxImpl::_notify_ack(const object_t& oid,
-				     uint64_t notify_id, uint64_t ver)
+int librados::IoCtxImpl::_notify_ack(
+  const object_t& oid,
+  uint64_t notify_id, uint64_t ver,
+  uint64_t cookie)
 {
   ::ObjectOperation rd;
   prepare_assert_ops(&rd);
-  rd.notify_ack(notify_id, ver);
+  rd.notify_ack(notify_id, ver, cookie);
   objecter->read(oid, oloc, rd, snap_seq, (bufferlist*)NULL, 0, 0, 0);
 
   return 0;
@@ -1521,7 +1493,7 @@ int librados::IoCtxImpl::notify(const object_t& oid, uint64_t ver, bufferlist& b
 
   lock->Lock();
   WatchContext *wc = new WatchContext(this, oid, ctx);
-  client->register_watcher(wc, oid, ctx, &cookie);
+  client->register_watcher(wc, &cookie);
   uint32_t prot_ver = 1;
   uint32_t timeout = notify_timeout;
   ::encode(prot_ver, inbl);
@@ -1609,6 +1581,33 @@ void librados::IoCtxImpl::C_aio_Ack::finish(int r)
   c->put_unlock();
 }
 
+///////////////////////////// C_aio_stat_Ack ////////////////////////////
+
+librados::IoCtxImpl::C_aio_stat_Ack::C_aio_stat_Ack(AioCompletionImpl *_c,
+						    time_t *pm)
+   : c(_c), pmtime(pm)
+{
+  c->get();
+}
+
+void librados::IoCtxImpl::C_aio_stat_Ack::finish(int r)
+{
+  c->lock.Lock();
+  c->rval = r;
+  c->ack = true;
+  c->cond.Signal();
+
+  if (r >= 0 && pmtime) {
+    *pmtime = mtime.sec();
+  }
+
+  if (c->callback_complete) {
+    c->io->client->finisher.queue(new C_AioComplete(c));
+  }
+
+  c->put_unlock();
+}
+
 /////////////////////// C_aio_sparse_read_Ack //////////////////////////
 
 librados::IoCtxImpl::C_aio_sparse_read_Ack::C_aio_sparse_read_Ack(AioCompletionImpl *_c,
@@ -1690,7 +1689,7 @@ void librados::IoCtxImpl::C_NotifyComplete::notify(uint8_t opcode,
 librados::WatchContext::WatchContext(IoCtxImpl *io_ctx_impl_,
 				     const object_t& _oc,
 				     librados::WatchCtx *_ctx)
-  : io_ctx_impl(io_ctx_impl_), oid(_oc), ctx(_ctx), linger_id(0)
+  : io_ctx_impl(io_ctx_impl_), oid(_oc), ctx(_ctx), linger_id(0), cookie(0)
 {
   io_ctx_impl->get();
 }
@@ -1709,7 +1708,7 @@ void librados::WatchContext::notify(Mutex *client_lock,
   ctx->notify(opcode, ver, payload);
   if (opcode != WATCH_NOTIFY_COMPLETE) {
     client_lock->Lock();
-    io_ctx_impl->_notify_ack(oid, notify_id, ver);
+    io_ctx_impl->_notify_ack(oid, notify_id, ver, cookie);
     client_lock->Unlock();
   }
 }
