@@ -224,7 +224,6 @@ namespace librbd {
 
   int ImageCtx::snap_set(string in_snap_name)
   {
-    assert(snap_lock.is_locked());
     map<string, SnapInfo>::iterator it = snaps_by_name.find(in_snap_name);
     if (it != snaps_by_name.end()) {
       snap_name = in_snap_name;
@@ -238,7 +237,6 @@ namespace librbd {
 
   void ImageCtx::snap_unset()
   {
-    assert(snap_lock.is_locked());
     snap_id = CEPH_NOSNAP;
     snap_name = "";
     snap_exists = true;
@@ -247,7 +245,6 @@ namespace librbd {
 
   snap_t ImageCtx::get_snap_id(string in_snap_name) const
   {
-    assert(snap_lock.is_locked());
     map<string, SnapInfo>::const_iterator it = snaps_by_name.find(in_snap_name);
     if (it != snaps_by_name.end())
       return it->second.id;
@@ -256,7 +253,6 @@ namespace librbd {
 
   int ImageCtx::get_snap_name(snapid_t in_snap_id, string *out_snap_name) const
   {
-    assert(snap_lock.is_locked());
     map<string, SnapInfo>::const_iterator it;
 
     for (it = snaps_by_name.begin(); it != snaps_by_name.end(); it++) {
@@ -270,7 +266,6 @@ namespace librbd {
 
   int ImageCtx::get_parent_spec(snapid_t in_snap_id, parent_spec *out_pspec)
   {
-    assert(snap_lock.is_locked());
     map<string, SnapInfo>::iterator it;
 
     for (it = snaps_by_name.begin(); it != snaps_by_name.end(); it++) {
@@ -322,7 +317,6 @@ namespace librbd {
 
   int ImageCtx::is_snap_protected(string in_snap_name, bool *is_protected) const
   {
-    assert(snap_lock.is_locked());
     map<string, SnapInfo>::const_iterator it = snaps_by_name.find(in_snap_name);
     if (it != snaps_by_name.end()) {
       *is_protected =
@@ -335,7 +329,6 @@ namespace librbd {
   int ImageCtx::is_snap_unprotected(string in_snap_name,
 				    bool *is_unprotected) const
   {
-    assert(snap_lock.is_locked());
     map<string, SnapInfo>::const_iterator it = snaps_by_name.find(in_snap_name);
     if (it != snaps_by_name.end()) {
       *is_unprotected =
@@ -345,23 +338,11 @@ namespace librbd {
     return -ENOENT;
   }
 
-  int ImageCtx::get_snap_size(string in_snap_name, uint64_t *out_size) const
-  {
-    assert(snap_lock.is_locked());
-    map<string, SnapInfo>::const_iterator it = snaps_by_name.find(in_snap_name);
-    if (it != snaps_by_name.end()) {
-      *out_size = it->second.size;
-      return 0;
-    }
-    return -ENOENT;
-  }
-
   void ImageCtx::add_snap(string in_snap_name, snap_t id, uint64_t in_size,
 			  uint64_t features,
 			  parent_info parent,
 			  uint8_t protection_status)
   {
-    assert(snap_lock.is_locked());
     snaps.push_back(id);
     SnapInfo info(id, in_size, features, parent, protection_status);
     snaps_by_name.insert(pair<string, SnapInfo>(in_snap_name, info));
@@ -369,8 +350,6 @@ namespace librbd {
 
   uint64_t ImageCtx::get_image_size(snap_t in_snap_id) const
   {
-    assert(md_lock.is_locked());
-    assert(snap_lock.is_locked());
     if (in_snap_id == CEPH_NOSNAP) {
       return size;
     }
@@ -386,8 +365,6 @@ namespace librbd {
 
   int ImageCtx::get_features(snap_t in_snap_id, uint64_t *out_features) const
   {
-    assert(md_lock.is_locked());
-    assert(snap_lock.is_locked());
     if (in_snap_id == CEPH_NOSNAP) {
       *out_features = features;
       return 0;
@@ -405,8 +382,6 @@ namespace librbd {
 
   int64_t ImageCtx::get_parent_pool_id(snap_t in_snap_id) const
   {
-    assert(snap_lock.is_locked());
-    assert(parent_lock.is_locked());
     if (in_snap_id == CEPH_NOSNAP) {
       return parent_md.spec.pool_id;
     }
@@ -422,8 +397,6 @@ namespace librbd {
 
   string ImageCtx::get_parent_image_id(snap_t in_snap_id) const
   {
-    assert(snap_lock.is_locked());
-    assert(parent_lock.is_locked());
     if (in_snap_id == CEPH_NOSNAP) {
       return parent_md.spec.image_id;
     }
@@ -439,8 +412,6 @@ namespace librbd {
 
   uint64_t ImageCtx::get_parent_snap_id(snap_t in_snap_id) const
   {
-    assert(snap_lock.is_locked());
-    assert(parent_lock.is_locked());
     if (in_snap_id == CEPH_NOSNAP) {
       return parent_md.spec.snap_id;
     }
@@ -456,8 +427,6 @@ namespace librbd {
 
   int ImageCtx::get_parent_overlap(snap_t in_snap_id, uint64_t *overlap) const
   {
-    assert(snap_lock.is_locked());
-    assert(parent_lock.is_locked());
     if (in_snap_id == CEPH_NOSNAP) {
       *overlap = parent_md.overlap;
       return 0;
@@ -475,9 +444,9 @@ namespace librbd {
 
   void ImageCtx::aio_read_from_cache(object_t o, bufferlist *bl, size_t len,
 				     uint64_t off, Context *onfinish) {
-    snap_lock.Lock();
+    snap_lock.get_read();
     ObjectCacher::OSDRead *rd = object_cacher->prepare_read(snap_id, bl, 0);
-    snap_lock.Unlock();
+    snap_lock.put_read();
     ObjectExtent extent(o, 0 /* a lie */, off, len);
     extent.oloc.pool = data_ctx.get_id();
     extent.buffer_extents.push_back(make_pair(0, len));
@@ -491,10 +460,10 @@ namespace librbd {
 
   void ImageCtx::write_to_cache(object_t o, bufferlist& bl, size_t len,
 				uint64_t off) {
-    snap_lock.Lock();
+    snap_lock.get_read();
     ObjectCacher::OSDWrite *wr = object_cacher->prepare_write(snapc, bl,
 							      utime_t(), 0);
-    snap_lock.Unlock();
+    snap_lock.put_read();
     ObjectExtent extent(o, 0, off, len);
     extent.oloc.pool = data_ctx.get_id();
     extent.buffer_extents.push_back(make_pair(0, len));
@@ -527,7 +496,7 @@ namespace librbd {
     bool done;
     Context *onfinish = new C_SafeCond(&mylock, &cond, &done, &r);
     cache_lock.Lock();
-    bool already_flushed = object_cacher->commit_set(object_set, onfinish);
+    bool already_flushed = object_cacher->flush_set(object_set, onfinish);
     cache_lock.Unlock();
     if (!already_flushed) {
       mylock.Lock();
@@ -542,14 +511,13 @@ namespace librbd {
   }
 
   void ImageCtx::shutdown_cache() {
-    md_lock.Lock();
+    md_lock.get_write();
     invalidate_cache();
-    md_lock.Unlock();
+    md_lock.put_write();
     object_cacher->stop();
   }
 
   void ImageCtx::invalidate_cache() {
-    assert(md_lock.is_locked());
     if (!object_cacher)
       return;
     cache_lock.Lock();
@@ -565,6 +533,14 @@ namespace librbd {
       lderr(cct) << "could not release all objects from cache" << dendl;
   }
 
+  void ImageCtx::clear_nonexistence_cache() {
+    if (!object_cacher)
+      return;
+    cache_lock.Lock();
+    object_cacher->clear_nonexistence(object_set);
+    cache_lock.Unlock();
+  }
+
   int ImageCtx::register_watch() {
     assert(!wctx);
     wctx = new WatchCtx(this);
@@ -582,8 +558,6 @@ namespace librbd {
   size_t ImageCtx::parent_io_len(uint64_t offset, size_t length,
 				 snap_t in_snap_id)
   {
-    assert(snap_lock.is_locked());
-    assert(parent_lock.is_locked());
     uint64_t overlap = 0;
     get_parent_overlap(in_snap_id, &overlap);
 
