@@ -27,7 +27,6 @@
 
 #include "include/filepath.h"
 
-#include "events/EString.h"
 #include "events/EExport.h"
 #include "events/EImportStart.h"
 #include "events/EImportFinish.h"
@@ -87,46 +86,46 @@ void Migrator::dispatch(Message *m)
   switch (m->get_type()) {
     // import
   case MSG_MDS_EXPORTDIRDISCOVER:
-    handle_export_discover((MExportDirDiscover*)m);
+    handle_export_discover(static_cast<MExportDirDiscover*>(m));
     break;
   case MSG_MDS_EXPORTDIRPREP:
-    handle_export_prep((MExportDirPrep*)m);
+    handle_export_prep(static_cast<MExportDirPrep*>(m));
     break;
   case MSG_MDS_EXPORTDIR:
-    handle_export_dir((MExportDir*)m);
+    handle_export_dir(static_cast<MExportDir*>(m));
     break;
   case MSG_MDS_EXPORTDIRFINISH:
-    handle_export_finish((MExportDirFinish*)m);
+    handle_export_finish(static_cast<MExportDirFinish*>(m));
     break;
   case MSG_MDS_EXPORTDIRCANCEL:
-    handle_export_cancel((MExportDirCancel*)m);
+    handle_export_cancel(static_cast<MExportDirCancel*>(m));
     break;
 
     // export 
   case MSG_MDS_EXPORTDIRDISCOVERACK:
-    handle_export_discover_ack((MExportDirDiscoverAck*)m);
+    handle_export_discover_ack(static_cast<MExportDirDiscoverAck*>(m));
     break;
   case MSG_MDS_EXPORTDIRPREPACK:
-    handle_export_prep_ack((MExportDirPrepAck*)m);
+    handle_export_prep_ack(static_cast<MExportDirPrepAck*>(m));
     break;
   case MSG_MDS_EXPORTDIRACK:
-    handle_export_ack((MExportDirAck*)m);
+    handle_export_ack(static_cast<MExportDirAck*>(m));
     break;
   case MSG_MDS_EXPORTDIRNOTIFYACK:
-    handle_export_notify_ack((MExportDirNotifyAck*)m);
+    handle_export_notify_ack(static_cast<MExportDirNotifyAck*>(m));
     break;    
 
     // export 3rd party (dir_auth adjustments)
   case MSG_MDS_EXPORTDIRNOTIFY:
-    handle_export_notify((MExportDirNotify*)m);
+    handle_export_notify(static_cast<MExportDirNotify*>(m));
     break;
 
     // caps
   case MSG_MDS_EXPORTCAPS:
-    handle_export_caps((MExportCaps*)m);
+    handle_export_caps(static_cast<MExportCaps*>(m));
     break;
   case MSG_MDS_EXPORTCAPSACK:
-    handle_export_caps_ack((MExportCapsAck*)m);
+    handle_export_caps_ack(static_cast<MExportCapsAck*>(m));
     break;
 
   default:
