@@ -12,41 +12,86 @@
  * 
  */
 
+#include <sstream>
 #include "Paxos.h"
 #include "Monitor.h"
-#include "MonitorStore.h"
+#include "MonitorDBStore.h"
 
 #include "messages/MMonPaxos.h"
 
 #include "common/config.h"
 #include "include/assert.h"
+#include "common/Formatter.h"
 
 #define dout_subsys ceph_subsys_paxos
 #undef dout_prefix
-#define dout_prefix _prefix(_dout, mon, mon->name, mon->rank, machine_name, state, first_committed, last_committed)
+#define dout_prefix _prefix(_dout, mon, mon->name, mon->rank, paxos_name, state, first_committed, last_committed)
 static ostream& _prefix(std::ostream *_dout, Monitor *mon, const string& name,
-		        int rank, const char *machine_name, int state,
+		        int rank, const string& paxos_name, int state,
 			version_t first_committed, version_t last_committed)
 {
   return *_dout << "mon." << name << "@" << rank
 		<< "(" << mon->get_state_name() << ")"
-		<< ".paxos(" << machine_name << " " << Paxos::get_statename(state)
+		<< ".paxos(" << paxos_name << " " << Paxos::get_statename(state)
 		<< " c " << first_committed << ".." << last_committed
 		<< ") ";
 }
 
+void Paxos::prepare_bootstrap()
+{
+  dout(0) << __func__ << dendl;
 
+  going_to_bootstrap = true;
+}
+
+MonitorDBStore *Paxos::get_store()
+{
+  return mon->store;
+}
+
+
+void Paxos::apply_version(MonitorDBStore::Transaction &tx, version_t v)
+{
+  bufferlist bl;
+  int err = get_store()->get(get_name(), v, bl);
+  assert(err == 0);
+  assert(bl.length());
+  decode_append_transaction(tx, bl);
+}
+
+void Paxos::reapply_all_versions()
+{
+  version_t first = get_store()->get(get_name(), "first_committed");
+  version_t last = get_store()->get(get_name(), "last_committed");
+  dout(10) << __func__ << " first " << first << " last " << last << dendl;
+
+  MonitorDBStore::Transaction tx;
+  for (version_t v = first; v <= last; ++v) {
+    dout(30) << __func__ << " apply version " << v << dendl;
+    apply_version(tx, v);
+  }
+  dout(15) << __func__ << " total versions " << (last-first) << dendl;
+
+  dout(30) << __func__ << " tx dump:\n";
+  JSONFormatter f(true);
+  tx.dump(&f);
+  f.flush(*_dout);
+  *_dout << dendl;
+
+  get_store()->apply_transaction(tx);
+}
 
 void Paxos::init()
 {
   // load paxos variables from stable storage
-  last_pn = mon->store->get_int(machine_name, "last_pn");
-  accepted_pn = mon->store->get_int(machine_name, "accepted_pn");
-  last_committed = mon->store->get_int(machine_name, "last_committed");
-  first_committed = mon->store->get_int(machine_name, "first_committed");
-  bufferlist temp;
-  latest_stashed = get_stashed(temp);
-  slurping = mon->store->get_int(machine_name, "slurping");
+  last_pn = get_store()->get(get_name(), "last_pn");
+  accepted_pn = get_store()->get(get_name(), "accepted_pn");
+  last_committed = get_store()->get(get_name(), "last_committed");
+  first_committed = get_store()->get(get_name(), "first_committed");
+
+  dout(10) << __func__ << " last_pn: " << last_pn << " accepted_pn: "
+	   << accepted_pn << " last_committed: " << last_committed
+	   << " first_committed: " << first_committed << dendl;
 
   dout(10) << "init" << dendl;
 }
@@ -70,10 +115,10 @@ void Paxos::collect(version_t oldpn)
   peer_last_committed.clear();
 
   // look for uncommitted value
-  if (mon->store->exists_bl_sn(machine_name, last_committed+1)) {
+  if (get_store()->exists(get_name(), last_committed+1)) {
     uncommitted_v = last_committed+1;
     uncommitted_pn = accepted_pn;
-    mon->store->get_bl_sn_safe(uncommitted_value, machine_name, last_committed+1);
+    get_store()->get(get_name(), last_committed+1, uncommitted_value);
     assert(uncommitted_value.length());
     dout(10) << "learned uncommitted " << (last_committed+1)
 	     << " (" << uncommitted_value.length() << " bytes) from myself" 
@@ -93,7 +138,7 @@ void Paxos::collect(version_t oldpn)
     if (*p == mon->rank) continue;
     
     MMonPaxos *collect = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_COLLECT,
-				       machine_id, ceph_clock_now(g_ceph_context));
+				       ceph_clock_now(g_ceph_context));
     collect->last_committed = last_committed;
     collect->first_committed = first_committed;
     collect->pn = accepted_pn;
@@ -116,8 +161,18 @@ void Paxos::handle_collect(MMonPaxos *collect)
   // we're recoverying, it seems!
   state = STATE_RECOVERING;
 
+  if (collect->first_committed > last_committed+1) {
+    dout(5) << __func__
+            << " leader's lowest version is too high for our last committed"
+            << " (theirs: " << collect->first_committed
+            << "; ours: " << last_committed << ") -- bootstrap!" << dendl;
+    collect->put();
+    mon->bootstrap();
+    return;
+  }
+
   // reply
-  MMonPaxos *last = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_LAST, machine_id,
+  MMonPaxos *last = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_LAST,
 				  ceph_clock_now(g_ceph_context));
   last->last_committed = last_committed;
   last->first_committed = first_committed;
@@ -129,7 +184,17 @@ void Paxos::handle_collect(MMonPaxos *collect)
     accepted_pn_from = collect->pn_from;
     dout(10) << "accepting pn " << accepted_pn << " from " 
 	     << accepted_pn_from << dendl;
-    mon->store->put_int(accepted_pn, machine_name, "accepted_pn");
+  
+    MonitorDBStore::Transaction t;
+    t.put(get_name(), "accepted_pn", accepted_pn);
+
+    dout(30) << __func__ << " transaction dump:\n";
+    JSONFormatter f(true);
+    t.dump(&f);
+    f.flush(*_dout);
+    *_dout << dendl;
+
+    get_store()->apply_transaction(t);
   } else {
     // don't accept!
     dout(10) << "NOT accepting pn " << collect->pn << " from " << collect->pn_from
@@ -146,13 +211,12 @@ void Paxos::handle_collect(MMonPaxos *collect)
   // do we have an accepted but uncommitted value?
   //  (it'll be at last_committed+1)
   bufferlist bl;
-  if (mon->store->exists_bl_sn(machine_name, last_committed+1)) {
-    mon->store->get_bl_sn_safe(bl, machine_name, last_committed+1);
+  if (get_store()->exists(get_name(), last_committed+1)) {
+    get_store()->get(get_name(), last_committed+1, bl);
     assert(bl.length() > 0);
     dout(10) << " sharing our accepted but uncommitted value for " 
 	     << last_committed+1 << " (" << bl.length() << " bytes)" << dendl;
     last->values[last_committed+1] = bl;
-    last->gv[last_committed+1] = mon->store->get_global_version(machine_name, last_committed+1);
     last->uncommitted_pn = accepted_pn;
   }
 
@@ -161,6 +225,18 @@ void Paxos::handle_collect(MMonPaxos *collect)
   collect->put();
 }
 
+/**
+ * @note This is Okay. We share our versions between peer_last_committed and
+ *	 our last_committed (inclusive), and add their bufferlists to the
+ *	 message. It will be the peer's job to apply them to his store, as
+ *	 these bufferlists will contain raw transactions.
+ *	 This function is called by both the Peon and the Leader. The Peon will
+ *	 share the state with the Leader during handle_collect(), sharing any
+ *	 values the leader may be missing (i.e., the leader's last_committed is
+ *	 lower than the peon's last_committed). The Leader will share the state
+ *	 with the Peon during handle_last(), if the peon's last_committed is
+ *	 lower than the leader's last_committed.
+ */
 void Paxos::share_state(MMonPaxos *m, version_t peer_first_committed,
 			version_t peer_last_committed)
 {
@@ -170,55 +246,36 @@ void Paxos::share_state(MMonPaxos *m, version_t peer_first_committed,
 	   << " lc " << peer_last_committed << dendl;
   version_t v = peer_last_committed + 1;
 
-  // start with a stashed full copy?
-  if (peer_last_committed + 1 < first_committed) {
-    bufferlist bl;
-    version_t l = get_stashed(bl);
-    assert(l <= last_committed);
-    dout(10) << "share_state starting with latest " << l 
-	     << " (" << bl.length() << " bytes)" << dendl;
-    m->latest_value.claim(bl);
-    m->latest_version = l;
-    v = l;
-  }
-
   // include incrementals
   for ( ; v <= last_committed; v++) {
-    if (mon->store->exists_bl_sn(machine_name, v)) {
-      mon->store->get_bl_sn_safe(m->values[v], machine_name, v);
+    if (get_store()->exists(get_name(), v)) {
+      get_store()->get(get_name(), v, m->values[v]);
       assert(m->values[v].length());
-      m->gv[v] = mon->store->get_global_version(machine_name, v);
-      dout(10) << " sharing " << v << " (" 
+      dout(10) << " sharing " << v << " ("
 	       << m->values[v].length() << " bytes)" << dendl;
-   }
+    }
   }
 
   m->last_committed = last_committed;
 }
 
+/**
+ * Store on disk a state that was shared with us
+ *
+ * Basically, we received a set of version. Or just one. It doesn't matter.
+ * What matters is that we have to stash it in the store. So, we will simply
+ * write every single bufferlist into their own versions on our side (i.e.,
+ * onto paxos-related keys), and then we will decode those same bufferlists
+ * we just wrote and apply the transactions they hold. We will also update
+ * our first and last committed values to point to the new values, if need
+ * be. All all this is done tightly wrapped in a transaction to ensure we
+ * enjoy the atomicity guarantees given by our awesome k/v store.
+ */
 void Paxos::store_state(MMonPaxos *m)
 {
+  MonitorDBStore::Transaction t;
   map<version_t,bufferlist>::iterator start = m->values.begin();
 
-  // stash?
-  if (m->latest_version && m->latest_version > last_committed) {
-    dout(10) << "store_state got stash version " 
-	     << m->latest_version << ", zapping old states" << dendl;
-
-    assert(start != m->values.end() && start->first == m->latest_version);
-
-    // wipe out everything we had previously
-    trim_to(last_committed + 1);
-
-    stash_latest(m->latest_version, m->latest_value);
-
-    first_committed = m->latest_version;
-    last_committed = m->latest_version;
-    mon->store->put_bl_sn(start->second, machine_name, m->latest_version);
-    mon->store->put_int(first_committed, machine_name, "first_committed");
-    mon->store->put_int(last_committed, machine_name, "last_committed");
-  }
-
   // build map of values to store
   // we want to write the range [last_committed, m->last_committed] only.
   if (start != m->values.end() &&
@@ -229,10 +286,14 @@ void Paxos::store_state(MMonPaxos *m)
     start = m->values.end();
   }
 
+  // push forward the start position on the message's values iterator, up until
+  // we run out of positions or we find a position matching 'last_committed'.
   while (start != m->values.end() && start->first <= last_committed) {
     ++start;
   }
 
+  // make sure we get the right interval of values to apply by pushing forward
+  // the 'end' iterator until it matches the message's 'last_committed'.
   map<version_t,bufferlist>::iterator end = start;
   while (end != m->values.end() && end->first <= m->last_committed) {
     last_committed = end->first;
@@ -246,13 +307,49 @@ void Paxos::store_state(MMonPaxos *m)
   } else {
     dout(10) << "store_state [" << start->first << ".." 
 	     << last_committed << "]" << dendl;
-
-    mon->store->put_bl_sn_map(machine_name, start, end, &m->gv);
-    mon->store->put_int(last_committed, machine_name, "last_committed");
-    mon->store->put_int(first_committed, machine_name, "first_committed");
+    t.put(get_name(), "last_committed", last_committed);
+    // we write our first_committed version before we append the message's
+    // transaction because this transaction may be a trim; if so, it will
+    // update the first_committed value, and we must let it clobber our own
+    // in order to obtain an updated state.
+    t.put(get_name(), "first_committed", first_committed);
+    // we should apply the state here -- decode every single bufferlist in the
+    // map and append the transactions to 't'.
+    map<version_t,bufferlist>::iterator it;
+    for (it = start; it != end; ++it) {
+      // write the bufferlist as the version's value
+      t.put(get_name(), it->first, it->second);
+      // decode the bufferlist and append it to the transaction we will shortly
+      // apply.
+      decode_append_transaction(t, it->second);
+    }
   }
-}
+  if (!t.empty()) {
+    dout(30) << __func__ << " transaction dump:\n";
+    JSONFormatter f(true);
+    t.dump(&f);
+    f.flush(*_dout);
+    *_dout << dendl;
 
+    get_store()->apply_transaction(t);
+
+    // update the first and last committed in-memory values.
+    first_committed = get_store()->get(get_name(), "first_committed");
+    last_committed = get_store()->get(get_name(), "last_committed");
+  }
+
+  if (get_store()->exists(get_name(), "conversion_first")) {
+    MonitorDBStore::Transaction scrub_tx;
+    version_t cf = get_store()->get(get_name(), "conversion_first");
+    dout(10) << __func__ << " scrub paxos from " << cf
+	     << " to " << first_committed << dendl;
+    if (cf < first_committed)
+      trim_to(&scrub_tx, cf, first_committed);
+    scrub_tx.erase(get_name(), "conversion_first");
+    get_store()->apply_transaction(scrub_tx);
+  }
+
+}
 
 // leader
 void Paxos::handle_last(MMonPaxos *last)
@@ -269,9 +366,19 @@ void Paxos::handle_last(MMonPaxos *last)
   // push it to them.
   peer_last_committed[last->get_source().num()] = last->last_committed;
 
-  // did we receive a committed value?
+  if (last->first_committed > last_committed+1) {
+    dout(5) << __func__
+            << " peon's lowest version is too high for our last committed"
+            << " (theirs: " << last->first_committed
+            << "; ours: " << last_committed << ") -- bootstrap!" << dendl;
+    last->put();
+    mon->bootstrap();
+    return;
+  }
+
+  // store any committed values if any are specified in the message
   store_state(last);
-      
+
   // do they accept your pn?
   if (last->pn > accepted_pn) {
     // no, try again.
@@ -289,13 +396,11 @@ void Paxos::handle_last(MMonPaxos *last)
 	     << num_last << " peons" << dendl;
 
     // did this person send back an accepted but uncommitted value?
-    version_t uncommitted_gv = 0;
     if (last->uncommitted_pn &&
 	last->uncommitted_pn > uncommitted_pn) {
       uncommitted_v = last->last_committed+1;
       uncommitted_pn = last->uncommitted_pn;
       uncommitted_value = last->values[uncommitted_v];
-      uncommitted_gv = last->gv[uncommitted_v];
       dout(10) << "we learned an uncommitted value for " << uncommitted_v 
 	       << " pn " << uncommitted_pn
 	       << " " << uncommitted_value.length() << " bytes"
@@ -316,7 +421,7 @@ void Paxos::handle_last(MMonPaxos *last)
 	  // share committed values
 	  dout(10) << " sending commit to mon." << p->first << dendl;
 	  MMonPaxos *commit = new MMonPaxos(mon->get_epoch(),
-					    MMonPaxos::OP_COMMIT, machine_id,
+					    MMonPaxos::OP_COMMIT,
 					    ceph_clock_now(g_ceph_context));
 	  share_state(commit, peer_first_committed[p->first], p->second);
 	  mon->messenger->send_message(commit, mon->monmap->get_inst(p->first));
@@ -326,26 +431,22 @@ void Paxos::handle_last(MMonPaxos *last)
       peer_last_committed.clear();
 
       // almost...
-      state = STATE_ACTIVE;
 
       // did we learn an old value?
       if (uncommitted_v == last_committed+1 &&
 	  uncommitted_value.length()) {
 	dout(10) << "that's everyone.  begin on old learned value" << dendl;
-	begin(uncommitted_value, uncommitted_gv);
+	state = STATE_PREPARING | STATE_LOCKED;
+	begin(uncommitted_value);
       } else {
 	// active!
+	state = STATE_ACTIVE;
 	dout(10) << "that's everyone.  active!" << dendl;
 	extend_lease();
 
-	// wake people up
-	if (mon->is_all_paxos_recovered()) {
-	  finish_contexts(g_ceph_context, waiting_for_active);
-	  finish_contexts(g_ceph_context, waiting_for_readable);
-	  finish_contexts(g_ceph_context, waiting_for_writeable);
-	} else {
-	  mon->recovered_leader(machine_id);
-	}
+	finish_contexts(g_ceph_context, waiting_for_active);
+	finish_contexts(g_ceph_context, waiting_for_readable);
+	finish_contexts(g_ceph_context, waiting_for_writeable);
       }
     }
   } else {
@@ -366,16 +467,16 @@ void Paxos::collect_timeout()
 
 
 // leader
-void Paxos::begin(bufferlist& v, version_t gv)
+void Paxos::begin(bufferlist& v)
 {
   dout(10) << "begin for " << last_committed+1 << " " 
 	   << v.length() << " bytes"
-	   << " gv " << gv
 	   << dendl;
 
   assert(mon->is_leader());
-  assert(is_active());
-  state = STATE_UPDATING;
+  assert(is_preparing());
+  state &= ~STATE_PREPARING;
+  state |= STATE_UPDATING;
 
   // we must already have a majority for this to work.
   assert(mon->get_quorum().size() == 1 ||
@@ -388,23 +489,34 @@ void Paxos::begin(bufferlist& v, version_t gv)
   accepted.clear();
   accepted.insert(mon->rank);
   new_value = v;
-  mon->store->put_bl_sn(new_value, machine_name, last_committed+1);
-  if (gv > 0)
-    mon->store->put_global_version(machine_name, last_committed+1, gv);
+  // store the proposed value in the store. IF it is accepted, we will then
+  // have to decode it into a transaction and apply it.
+  MonitorDBStore::Transaction t;
+  t.put(get_name(), last_committed+1, new_value);
+
+  dout(30) << __func__ << " transaction dump:\n";
+  JSONFormatter f(true);
+  t.dump(&f);
+  f.flush(*_dout);
+  MonitorDBStore::Transaction debug_tx;
+  bufferlist::iterator new_value_it = new_value.begin();
+  debug_tx.decode(new_value_it);
+  debug_tx.dump(&f);
+  *_dout << "\nbl dump:\n";
+  f.flush(*_dout);
+  *_dout << dendl;
+
+  get_store()->apply_transaction(t);
 
   if (mon->get_quorum().size() == 1) {
     // we're alone, take it easy
     commit();
     state = STATE_ACTIVE;
-
-    if (mon->is_all_paxos_recovered()) {
-      finish_contexts(g_ceph_context, waiting_for_active);
-      finish_contexts(g_ceph_context, waiting_for_commit);
-      finish_contexts(g_ceph_context, waiting_for_readable);
-      finish_contexts(g_ceph_context, waiting_for_writeable);
-    } else {
-      mon->recovered_leader(machine_id);
-    }
+    finish_proposal();
+    finish_contexts(g_ceph_context, waiting_for_active);
+    finish_contexts(g_ceph_context, waiting_for_commit);
+    finish_contexts(g_ceph_context, waiting_for_readable);
+    finish_contexts(g_ceph_context, waiting_for_writeable);
 
     return;
   }
@@ -417,9 +529,8 @@ void Paxos::begin(bufferlist& v, version_t gv)
     
     dout(10) << " sending begin to mon." << *p << dendl;
     MMonPaxos *begin = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_BEGIN,
-				     machine_id, ceph_clock_now(g_ceph_context));
+				     ceph_clock_now(g_ceph_context));
     begin->values[last_committed+1] = new_value;
-    begin->gv[last_committed+1] = gv;
     begin->last_committed = last_committed;
     begin->pn = accepted_pn;
     
@@ -451,14 +562,23 @@ void Paxos::handle_begin(MMonPaxos *begin)
 
   // yes.
   version_t v = last_committed+1;
-  dout(10) << "accepting value for " << v << " pn " << accepted_pn << " gv " << begin->gv[v] << dendl;
-  mon->store->put_bl_sn(begin->values[v], machine_name, v);
-  if (begin->gv.count(v) && begin->gv[v] > 0)
-    mon->store->put_global_version(machine_name, v, begin->gv[v]);
-  
+  dout(10) << "accepting value for " << v << " pn " << accepted_pn << dendl;
+  // store the accepted value onto our store. We will have to decode it and
+  // apply its transaction once we receive permission to commit.
+  MonitorDBStore::Transaction t;
+  t.put(get_name(), v, begin->values[v]);
+
+  dout(30) << __func__ << " transaction dump:\n";
+  JSONFormatter f(true);
+  t.dump(&f);
+  f.flush(*_dout);
+  *_dout << dendl;
+
+  get_store()->apply_transaction(t);
+
   // reply
   MMonPaxos *accept = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_ACCEPT,
-				    machine_id, ceph_clock_now(g_ceph_context));
+				    ceph_clock_now(g_ceph_context));
   accept->pn = accepted_pn;
   accept->last_committed = last_committed;
   mon->messenger->send_message(accept, begin->get_source_inst());
@@ -487,7 +607,7 @@ void Paxos::handle_accept(MMonPaxos *accept)
   assert(accept->last_committed == last_committed ||   // not committed
 	 accept->last_committed == last_committed-1);  // committed
 
-  assert(state == STATE_UPDATING);
+  assert(is_updating());
   assert(accepted.count(from) == 0);
   accepted.insert(from);
   dout(10) << " now " << accepted << " have accepted" << dendl;
@@ -510,16 +630,14 @@ void Paxos::handle_accept(MMonPaxos *accept)
     // yay!
     state = STATE_ACTIVE;
     extend_lease();
+  
+    finish_proposal();
 
     // wake people up
-    if (mon->is_all_paxos_recovered()) {
-      finish_contexts(g_ceph_context, waiting_for_active);
-      finish_contexts(g_ceph_context, waiting_for_commit);
-      finish_contexts(g_ceph_context, waiting_for_readable);
-      finish_contexts(g_ceph_context, waiting_for_writeable);
-    } else {
-      mon->recovered_leader(machine_id);
-    }
+    finish_contexts(g_ceph_context, waiting_for_active);
+    finish_contexts(g_ceph_context, waiting_for_commit);
+    finish_contexts(g_ceph_context, waiting_for_readable);
+    finish_contexts(g_ceph_context, waiting_for_writeable);
   }
   accept->put();
 }
@@ -542,15 +660,29 @@ void Paxos::commit()
   //   leader still got a majority and committed with out us.)
   lease_expire = utime_t();  // cancel lease
 
+  MonitorDBStore::Transaction t;
+
   // commit locally
   last_committed++;
   last_commit_time = ceph_clock_now(g_ceph_context);
-  mon->store->put_int(last_committed, machine_name, "last_committed");
+  t.put(get_name(), "last_committed", last_committed);
   if (!first_committed) {
     first_committed = last_committed;
-    mon->store->put_int(last_committed, machine_name, "first_committed");
+    t.put(get_name(), "first_committed", last_committed);
   }
 
+  // decode the value and apply its transaction to the store.
+  // this value can now be read from last_committed.
+  decode_append_transaction(t, new_value);
+
+  dout(30) << __func__ << " transaction dump:\n";
+  JSONFormatter f(true);
+  t.dump(&f);
+  f.flush(*_dout);
+  *_dout << dendl;
+
+  get_store()->apply_transaction(t);
+
   // tell everyone
   for (set<int>::const_iterator p = mon->get_quorum().begin();
        p != mon->get_quorum().end();
@@ -559,9 +691,8 @@ void Paxos::commit()
 
     dout(10) << " sending commit to mon." << *p << dendl;
     MMonPaxos *commit = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_COMMIT,
-				      machine_id, ceph_clock_now(g_ceph_context));
+				      ceph_clock_now(g_ceph_context));
     commit->values[last_committed] = new_value;
-    commit->gv[last_committed] = mon->store->get_global_version(machine_name, last_committed);
     commit->pn = accepted_pn;
     commit->last_committed = last_committed;
 
@@ -588,9 +719,7 @@ void Paxos::handle_commit(MMonPaxos *commit)
   
   commit->put();
 
-  if (mon->is_all_paxos_recovered())
-    finish_contexts(g_ceph_context, waiting_for_commit);
-  // otherwise, this'll go when they all recover.
+  finish_contexts(g_ceph_context, waiting_for_commit);
 }
 
 void Paxos::extend_lease()
@@ -612,7 +741,7 @@ void Paxos::extend_lease()
 
     if (*p == mon->rank) continue;
     MMonPaxos *lease = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_LEASE,
-				     machine_id, ceph_clock_now(g_ceph_context));
+				     ceph_clock_now(g_ceph_context));
     lease->last_committed = last_committed;
     lease->lease_timestamp = lease_expire;
     lease->first_committed = first_committed;
@@ -654,6 +783,71 @@ void Paxos::warn_on_future_time(utime_t t, entity_name_t from)
 
 }
 
+void Paxos::finish_queued_proposal()
+{
+  assert(mon->is_leader());
+  assert(!proposals.empty());
+
+  dout(10) << __func__ << " finishing proposal" << dendl;
+  C_Proposal *proposal = static_cast<C_Proposal*>(proposals.front());
+  dout(10) << __func__ << " finish it (proposal = "
+	   << proposal << ")" << dendl;;
+
+  assert(proposal != NULL);
+
+  if (!proposal->proposed) {
+    dout(10) << __func__ << " we must have received a stay message and we're "
+             << "trying to finish before time. "
+	     << "Instead, propose it (if we are active)!" << dendl;
+  } else {
+    dout(10) << __func__ << " proposal took "
+	     << (ceph_clock_now(NULL) - proposal->proposal_time)
+	     << " to finish" << dendl;
+
+    proposals.pop_front();
+    proposal->finish(0);
+  }
+}
+
+void Paxos::finish_proposal()
+{
+  /* There is a lot of debug still going around. We will get rid of it later
+   * on, as soon as everything "just works (tm)"
+   */
+  assert(mon->is_leader());
+  if (!proposals.empty())
+    finish_queued_proposal();
+
+  dout(10) << __func__ << " state " << state
+	   << " proposals left " << proposals.size() << dendl;
+
+  /* Update the internal Paxos state.
+   *
+   * Since we moved to a single-paxos instance across all monitor services, we
+   * can no longer rely on each individual service to update paxos state.
+   * Therefore, once we conclude a proposal, we must update our internal
+   * state (say, such variables as 'first_committed'), because no one else
+   * will take care of that for us -- and we rely on these variables for
+   * several other mechanisms; trimming comes to mind.
+   */
+  first_committed = get_store()->get(get_name(), "first_committed");
+  last_committed = get_store()->get(get_name(), "last_committed");
+
+  if (proposals.empty() && going_to_bootstrap) {
+    dout(0) << __func__ << " no more proposals; bootstraping." << dendl;
+    mon->bootstrap();
+    return;
+  }
+
+  if (should_trim()) {
+    trim();
+  }
+
+  if (is_active() && !proposals.empty()) {
+    propose_queued();
+  }
+}
+
 // peon
 void Paxos::handle_lease(MMonPaxos *lease)
 {
@@ -682,15 +876,12 @@ void Paxos::handle_lease(MMonPaxos *lease)
 
   state = STATE_ACTIVE;
 
-  if (!mon->is_all_paxos_recovered())
-    mon->recovered_peon(machine_id);
-
   dout(10) << "handle_lease on " << lease->last_committed
 	   << " now " << lease_expire << dendl;
 
   // ack
   MMonPaxos *ack = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_LEASE_ACK,
-				 machine_id, ceph_clock_now(g_ceph_context));
+				 ceph_clock_now(g_ceph_context));
   ack->last_committed = last_committed;
   ack->first_committed = first_committed;
   ack->lease_timestamp = ceph_clock_now(g_ceph_context);
@@ -702,9 +893,6 @@ void Paxos::handle_lease(MMonPaxos *lease)
   lease_timeout_event = new C_LeaseTimeout(this);
   mon->timer.add_event_after(g_conf->mon_lease_ack_timeout, lease_timeout_event);
 
-  // trim?
-  trim_to(lease->first_committed);
-  
   // kick waiters
   finish_contexts(g_ceph_context, waiting_for_active);
   if (is_readable())
@@ -730,6 +918,8 @@ void Paxos::handle_lease_ack(MMonPaxos *ack)
 	       << " -- got everyone" << dendl;
       mon->timer.cancel_event(lease_ack_timeout_event);
       lease_ack_timeout_event = 0;
+
+
     } else {
       dout(10) << "handle_lease_ack from " << ack->get_source() 
 	       << " -- still need "
@@ -775,27 +965,58 @@ void Paxos::lease_renew_timeout()
 /*
  * trim old states
  */
+void Paxos::trim_to(MonitorDBStore::Transaction *t,
+		    version_t from, version_t to) {
+  dout(10) << __func__ << " from " << from << " to " << to << dendl;
+  assert(from < to);
 
-void Paxos::trim_to(version_t first, bool force)
+  while (from < to) {
+    dout(10) << "trim " << from << dendl;
+    t->erase(get_name(), from);
+    from++;
+  }
+}
+
+void Paxos::trim_to(MonitorDBStore::Transaction *t, version_t first)
 {
   dout(10) << "trim_to " << first << " (was " << first_committed << ")"
-	   << ", latest_stashed " << latest_stashed
 	   << dendl;
 
   if (first_committed >= first)
     return;
+  trim_to(t, first_committed, first);
+  t->put(get_name(), "first_committed", first);
+}
 
-  while (first_committed < first &&
-	 (force || first_committed < latest_stashed)) {
-    dout(10) << "trim " << first_committed << dendl;
-    mon->store->erase_sn(machine_name, first_committed);
-    for (list<string>::iterator p = extra_state_dirs.begin();
-	 p != extra_state_dirs.end();
-	 ++p)
-      mon->store->erase_sn(p->c_str(), first_committed);
-    first_committed++;
+void Paxos::trim_to(version_t first)
+{
+  MonitorDBStore::Transaction t;
+  
+  trim_to(&t, first);
+
+  if (!t.empty()) {
+    dout(30) << __func__ << " transaction dump:\n";
+    JSONFormatter f(true);
+    t.dump(&f);
+    f.flush(*_dout);
+    *_dout << dendl;
+
+    bufferlist bl;
+    t.encode(bl);
+
+    going_to_trim = true;
+    queue_proposal(bl, new C_Trimmed(this));
   }
-  mon->store->put_int(first_committed, machine_name, "first_committed");
+}
+
+void Paxos::trim_enable() {
+  trim_disabled_version = 0;
+  // We may not be the leader when we reach this function. We sure must
+  // have been the leader at some point, but we may have been demoted and
+  // we really should reset 'trim_disabled_version' if that was the case.
+  // So, make sure we only trim() iff we are the leader.
+  if (mon->is_leader() && should_trim())
+    trim();
 }
 
 /*
@@ -813,7 +1034,16 @@ version_t Paxos::get_new_proposal_number(version_t gt)
   last_pn += (version_t)mon->rank;
   
   // write
-  mon->store->put_int(last_pn, machine_name, "last_pn");
+  MonitorDBStore::Transaction t;
+  t.put(get_name(), "last_pn", last_pn);
+
+  dout(30) << __func__ << " transaction dump:\n";
+  JSONFormatter f(true);
+  t.dump(&f);
+  f.flush(*_dout);
+  *_dout << dendl;
+
+  get_store()->apply_transaction(t);
 
   dout(10) << "get_new_proposal_number = " << last_pn << dendl;
   return last_pn;
@@ -856,10 +1086,13 @@ void Paxos::leader_init()
 {
   cancel_events();
   new_value.clear();
+  if (!proposals.empty())
+    proposals.clear();
+
+  going_to_bootstrap = false;
 
   if (mon->get_quorum().size() == 1) {
     state = STATE_ACTIVE;
-    mon->recovered_leader(machine_id);
     return;
   }
 
@@ -888,6 +1121,12 @@ void Paxos::restart()
   dout(10) << "restart -- canceling timeouts" << dendl;
   cancel_events();
   new_value.clear();
+  dout(10) << __func__ << " -- clearing queued proposals" << dendl;
+  if (!proposals.empty())
+    proposals.clear();
+
+  state = STATE_RECOVERING;
+  going_to_bootstrap = false;
 
   finish_contexts(g_ceph_context, waiting_for_commit, -EAGAIN);
   finish_contexts(g_ceph_context, waiting_for_active, -EAGAIN);
@@ -965,12 +1204,12 @@ bool Paxos::is_readable(version_t v)
     (is_active() || is_updating()) &&
     last_committed > 0 &&           // must have a value
     (mon->get_quorum().size() == 1 ||  // alone, or
-     ceph_clock_now(g_ceph_context) < lease_expire);    // have lease
+     is_lease_valid()); // have lease
 }
 
 bool Paxos::read(version_t v, bufferlist &bl)
 {
-  if (mon->store->get_bl_sn(bl, machine_name, v) <= 0)
+  if (!get_store()->get(get_name(), v, bl))
     return false;
   return true;
 }
@@ -983,121 +1222,81 @@ version_t Paxos::read_current(bufferlist &bl)
 }
 
 
-
+bool Paxos::is_lease_valid()
+{
+  return ((mon->get_quorum().size() == 1)
+      || (ceph_clock_now(g_ceph_context) < lease_expire));
+}
 
 // -- WRITE --
 
 bool Paxos::is_writeable()
 {
-  // do not allow new paxos writes until all paxos machines have
-  // recovered.  this ensures that the global versions we choose at
-  // proposal time are sanely ordered.
-  if (!mon->is_all_paxos_recovered())
-    return false;
-
-  if (mon->get_quorum().size() == 1) return true;
   return
     mon->is_leader() &&
     is_active() &&
-    ceph_clock_now(g_ceph_context) < lease_expire;
+    is_lease_valid();
 }
 
-bool Paxos::propose_new_value(bufferlist& bl, Context *oncommit)
+void Paxos::list_proposals(ostream& out)
 {
-  /*
-  // writeable?
-  if (!is_writeable()) {
-    dout(5) << "propose_new_value " << last_committed+1 << " " << bl.length() << " bytes"
-	    << " -- not writeable" << dendl;
-    if (oncommit) {
-      oncommit->finish(-1);
-      delete oncommit;
-    }
-    return false;
+  out << __func__ << " " << proposals.size() << " in queue:\n";
+  list<Context*>::iterator p_it = proposals.begin();
+  for (int i = 0; p_it != proposals.end(); ++p_it, ++i) {
+    C_Proposal *p = (C_Proposal*) *p_it;
+    out << "-- entry #" << i << "\n";
+    out << *p << "\n";
   }
-  */
-  
-  assert(mon->is_leader() && is_active());
+}
+
+void Paxos::propose_queued()
+{
+  assert(is_active());
+  assert(!proposals.empty());
+
+  state = STATE_PREPARING;
+
+  C_Proposal *proposal = static_cast<C_Proposal*>(proposals.front());
+  assert(!proposal->proposed);
 
-  // cancel lease renewal and timeout events.
   cancel_events();
+  dout(5) << __func__ << " " << (last_committed + 1)
+	  << " " << proposal->bl.length() << " bytes" << dendl;
+  proposal->proposed = true;
 
-  version_t global_version = mon->get_global_paxos_version();
+  dout(10) << __func__ << " ";
+  list_proposals(*_dout);
+  *_dout << dendl;
 
-  // ok!
-  dout(5) << "propose_new_value " << last_committed+1 << " " << bl.length() << " bytes, gv " << global_version << dendl;
-  if (oncommit)
-    waiting_for_commit.push_back(oncommit);
-  begin(bl, global_version);
+  begin(proposal->bl);
+}
+
+void Paxos::queue_proposal(bufferlist& bl, Context *onfinished)
+{
+  dout(5) << __func__ << " bl " << bl.length() << " bytes;"
+	  << " ctx = " << onfinished << dendl;
+
+  proposals.push_back(new C_Proposal(onfinished, bl));
+}
+
+bool Paxos::propose_new_value(bufferlist& bl, Context *onfinished)
+{
+  assert(mon->is_leader());
+
+  queue_proposal(bl, onfinished);
+
+  if (!is_active()) {
+    dout(5) << __func__ << " not active; proposal queued" << dendl; 
+    return true;
+  }
+
+  propose_queued();
   
   return true;
 }
 
-void Paxos::stash_latest(version_t v, bufferlist& bl)
-{
-  if (v == latest_stashed) {
-    dout(10) << "stash_latest v" << v << " already stashed" << dendl;
-    return;  // already stashed.
-  }
-
-  bufferlist final;
-  ::encode(v, final);
-  ::encode(bl, final);
-  
-  dout(10) << "stash_latest v" << v << " len " << bl.length() << dendl;
-  mon->store->put_bl_ss(final, machine_name, "latest");
-
-  latest_stashed = v;
-}
-
-version_t Paxos::get_stashed(bufferlist& bl)
-{
-  bufferlist full;
-  mon->store->get_bl_ss_safe(full, machine_name, "latest");
-  if (!full.length()) {
-    dout(10) << "get_stashed not found" << dendl;
-    return 0;
-  }
-  bufferlist::iterator p = full.begin();
-  version_t v;
-  ::decode(v, p);
-  ::decode(bl, p);
-
-  latest_stashed = v;
-  dout(10) << "get_stashed v" << latest_stashed << " len " << bl.length() << dendl;
-  return latest_stashed;  
-}
-
-
 bool Paxos::is_consistent()
 {
-  bool consistent = true;
-  if (first_committed > 1 && first_committed > latest_stashed)
-    consistent = false;
-  if (first_committed > last_committed)
-    consistent = false;
-  if (latest_stashed > last_committed)
-    consistent = false;
-  if (slurping != 0)
-    consistent = false;
-
-  assert(consistent || (slurping == 1));
-  return consistent;
+  return (first_committed <= last_committed);
 }
 
-void Paxos::start_slurping()
-{
-  if (slurping != 1) {
-    slurping = 1;
-    mon->store->put_int(1, machine_name, "slurping");
-  }
-}
-
-void Paxos::end_slurping()
-{
-  if (slurping == 1) {
-    slurping = 0;
-    mon->store->put_int(slurping, machine_name, "slurping");
-  }
-  assert(is_consistent());
-}
