@@ -1,4 +1,5 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
 #ifndef LEVEL_DB_STORE_H
 #define LEVEL_DB_STORE_H
 
@@ -20,11 +21,20 @@
 class LevelDBStore : public KeyValueDB {
   string path;
   boost::scoped_ptr<leveldb::DB> db;
+
+  int init(ostream &out, bool create_if_missing);
+
 public:
   LevelDBStore(const string &path) : path(path) {}
 
   /// Opens underlying db
-  int init(ostream &out);
+  int open(ostream &out) {
+    return init(out, false);
+  }
+  /// Creates underlying db if missing and opens it
+  int create_and_open(ostream &out) {
+    return init(out, true);
+  }
 
   class LevelDBTransactionImpl : public KeyValueDB::TransactionImpl {
   public:
@@ -36,12 +46,11 @@ public:
     LevelDBTransactionImpl(LevelDBStore *db) : db(db) {}
     void set(
       const string &prefix,
-      const std::map<string, bufferlist> &to_set
-      );
-    void rmkeys(
+      const string &k,
+      const bufferlist &bl);
+    void rmkey(
       const string &prefix,
-      const std::set<string> &keys
-      );
+      const string &k);
     void rmkeys_by_prefix(
       const string &prefix
       );
@@ -74,42 +83,57 @@ public:
     std::map<string, bufferlist> *out
     );
 
-  class LevelDBIteratorImpl : public KeyValueDB::IteratorImpl {
+  class LevelDBWholeSpaceIteratorImpl :
+    public KeyValueDB::WholeSpaceIteratorImpl {
+  protected:
     boost::scoped_ptr<leveldb::Iterator> dbiter;
-    const string prefix;
   public:
-    LevelDBIteratorImpl(leveldb::Iterator *iter, const string &prefix) :
-      dbiter(iter), prefix(prefix) {}
+    LevelDBWholeSpaceIteratorImpl(leveldb::Iterator *iter) :
+      dbiter(iter) { }
+    virtual ~LevelDBWholeSpaceIteratorImpl() { }
+
     int seek_to_first() {
+      dbiter->SeekToFirst();
+      return dbiter->status().ok() ? 0 : -1;
+    }
+    int seek_to_first(const string &prefix) {
       leveldb::Slice slice_prefix(prefix);
       dbiter->Seek(slice_prefix);
       return dbiter->status().ok() ? 0 : -1;
     }
     int seek_to_last() {
+      dbiter->SeekToLast();
+      return dbiter->status().ok() ? 0 : -1;
+    }
+    int seek_to_last(const string &prefix) {
       string limit = past_prefix(prefix);
       leveldb::Slice slice_limit(limit);
-	dbiter->Seek(slice_limit);
+      dbiter->Seek(slice_limit);
+
       if (!dbiter->Valid()) {
-	dbiter->SeekToLast();
+        dbiter->SeekToLast();
       } else {
-	dbiter->Prev();
+        dbiter->Prev();
       }
       return dbiter->status().ok() ? 0 : -1;
     }
-    int upper_bound(const string &after) {
-      lower_bound(after);
-      if (valid() && key() == after)
-	next();
+    int upper_bound(const string &prefix, const string &after) {
+      lower_bound(prefix, after);
+      if (valid()) {
+	pair<string,string> key = raw_key();
+	if (key.first == prefix && key.second == after)
+	  next();
+      }
       return dbiter->status().ok() ? 0 : -1;
     }
-    int lower_bound(const string &to) {
+    int lower_bound(const string &prefix, const string &to) {
       string bound = combine_strings(prefix, to);
       leveldb::Slice slice_bound(bound);
       dbiter->Seek(slice_bound);
       return dbiter->status().ok() ? 0 : -1;
     }
     bool valid() {
-      return dbiter->Valid() && in_prefix(prefix, dbiter->key());
+      return dbiter->Valid();
     }
     int next() {
       if (valid())
@@ -126,6 +150,11 @@ public:
       split_key(dbiter->key(), 0, &out_key);
       return out_key;
     }
+    pair<string,string> raw_key() {
+      string prefix, key;
+      split_key(dbiter->key(), &prefix, &key);
+      return make_pair(prefix, key);
+    }
     bufferlist value() {
       return to_bufferlist(dbiter->value());
     }
@@ -133,13 +162,20 @@ public:
       return dbiter->status().ok() ? 0 : -1;
     }
   };
-  Iterator get_iterator(const string &prefix) {
-    return std::tr1::shared_ptr<LevelDBIteratorImpl>(
-      new LevelDBIteratorImpl(
-	db->NewIterator(leveldb::ReadOptions()),
-	prefix));
-  }
 
+  class LevelDBSnapshotIteratorImpl : public LevelDBWholeSpaceIteratorImpl {
+    leveldb::DB *db;
+    const leveldb::Snapshot *snapshot;
+  public:
+    LevelDBSnapshotIteratorImpl(leveldb::DB *db, const leveldb::Snapshot *s,
+				leveldb::Iterator *iter) :
+      LevelDBWholeSpaceIteratorImpl(iter), db(db), snapshot(s) { }
+
+    ~LevelDBSnapshotIteratorImpl() {
+      assert(snapshot != NULL);
+      db->ReleaseSnapshot(snapshot);
+    }
+  };
 
   /// Utility
   static string combine_strings(const string &prefix, const string &value);
@@ -149,11 +185,34 @@ public:
     return (key.compare(leveldb::Slice(past_prefix(prefix))) < 0) &&
       (key.compare(leveldb::Slice(prefix)) > 0);
   }
-  static string past_prefix(const string prefix) {
+  static string past_prefix(const string &prefix) {
     string limit = prefix;
     limit.push_back(1);
     return limit;
   }
+
+protected:
+  WholeSpaceIterator _get_iterator() {
+    return std::tr1::shared_ptr<KeyValueDB::WholeSpaceIteratorImpl>(
+      new LevelDBWholeSpaceIteratorImpl(
+	db->NewIterator(leveldb::ReadOptions())
+      )
+    );
+  }
+
+  WholeSpaceIterator _get_snapshot_iterator() {
+    const leveldb::Snapshot *snapshot;
+    leveldb::ReadOptions options;
+
+    snapshot = db->GetSnapshot();
+    options.snapshot = snapshot;
+
+    return std::tr1::shared_ptr<KeyValueDB::WholeSpaceIteratorImpl>(
+      new LevelDBSnapshotIteratorImpl(db.get(), snapshot,
+	db->NewIterator(options))
+    );
+  }
+
 };
 
 #endif
