@@ -834,6 +834,7 @@ public:
       block_writes(false), active(false), queue_snap_trim(false),
       waiting_on(0), errors(0), fixed(0), active_rep_scrub(0),
       must_scrub(false), must_deep_scrub(false), must_repair(false),
+      classic(false),
       finalizing(false), is_chunky(false), state(INACTIVE),
       deep(false)
     {
@@ -869,6 +870,7 @@ public:
     map<hobject_t, pair<ScrubMap::object, int> > authoritative;
 
     // classic scrub
+    bool classic;
     bool finalizing;
 
     // chunky scrub
@@ -940,6 +942,7 @@ public:
 
     // clear all state
     void reset() {
+      classic = false;
       finalizing = false;
       block_writes = false;
       active = false;
@@ -1868,7 +1871,7 @@ public:
   bool old_peering_evt(CephPeeringEvtRef evt) {
     return old_peering_msg(evt->get_epoch_sent(), evt->get_epoch_requested());
   }
-  bool require_same_or_newer_map(epoch_t e) {
+  bool have_same_or_newer_map(epoch_t e) {
     return e <= get_osdmap()->get_epoch();
   }
 
@@ -1920,17 +1923,8 @@ public:
   virtual void on_role_change() = 0;
   virtual void on_change() = 0;
   virtual void on_activate() = 0;
+  virtual void on_flushed() = 0;
   virtual void on_shutdown() = 0;
-  virtual void remove_watchers_and_notifies() = 0;
-
-  virtual void register_unconnected_watcher(void *obc,
-					    entity_name_t entity,
-					    utime_t expire) = 0;
-  virtual void unregister_unconnected_watcher(void *obc,
-					      entity_name_t entity) = 0;
-  virtual void handle_watch_timeout(void *obc,
-				    entity_name_t entity,
-				    utime_t expire) = 0;
 };
 
 WRITE_CLASS_ENCODER(PG::OndiskLog)
