@@ -287,6 +287,9 @@ void Objecter::send_linger(LingerOp *info)
     info->register_tid = _op_submit(o);
   } else {
     // first send
+    // populate info->pgid and info->acting so we
+    // don't resend the linger op on the next osdmap update
+    recalc_linger_op_target(info);
     info->register_tid = op_submit(o);
   }
 
@@ -580,10 +583,10 @@ void Objecter::handle_osd_map(MOSDMap *m)
   // was/is paused?
   if (was_pauserd || was_pausewr || pauserd || pausewr)
     maybe_request_map();
-
+  
   // unpause requests?
   if ((was_pauserd && !pauserd) ||
-      (was_pausewr && !pausewr)) {
+      (was_pausewr && !pausewr))
     for (map<tid_t,Op*>::iterator p = ops.begin();
 	 p != ops.end();
 	 p++) {
@@ -593,16 +596,6 @@ void Objecter::handle_osd_map(MOSDMap *m)
 	  !((op->flags & CEPH_OSD_FLAG_WRITE) && pausewr))    // not still paused as a write
 	need_resend[op->tid] = op;
     }
-    for (map<tid_t, LingerOp*>::iterator lp = linger_ops.begin();
-	 lp != linger_ops.end();
-	 ++lp) {
-      LingerOp *op = lp->second;
-      if (!op->registered &&
-	  !pauserd &&                                      // not still paused as a read
-	  !((op->flags & CEPH_OSD_FLAG_WRITE) && pausewr)) // not still paused as a write
-	need_resend_linger.push_back(op);
-    }
-  }
 
   // resend requests
   for (map<tid_t, Op*>::iterator p = need_resend.begin(); p != need_resend.end(); p++) {
@@ -1029,10 +1022,9 @@ tid_t Objecter::_op_submit(Op *op)
   assert(client_inc >= 0);
 
   // pick target
-  bool check_for_latest_map = false;
   num_homeless_ops++;  // initially; recalc_op_target() will decrement if it finds a target
   int r = recalc_op_target(op);
-  check_for_latest_map = (r == RECALC_OP_TARGET_POOL_DNE);
+  bool check_for_latest_map = (r == RECALC_OP_TARGET_POOL_DNE);
 
   // add to gather set(s)
   if (op->onack) {
@@ -1167,7 +1159,7 @@ int Objecter::recalc_op_target(Op *op)
 
     OSDSession *s = NULL;
     op->used_replica = false;
-    if (acting.size()) {
+    if (!acting.empty()) {
       int osd;
       bool read = (op->flags & CEPH_OSD_FLAG_READ) && (op->flags & CEPH_OSD_FLAG_WRITE) == 0;
       if (read && (op->flags & CEPH_OSD_FLAG_BALANCE_READS)) {
