@@ -707,18 +707,16 @@ static void check_bad_user_bucket_mapping(RGWRados *store, const string& user_id
 
 static int remove_object(RGWRados *store, rgw_bucket& bucket, std::string& object)
 {
-  int ret = -EINVAL;
   RGWRadosCtx *rctx = new RGWRadosCtx(store);
   rgw_obj obj(bucket,object);
 
-  ret = store->delete_obj(rctx, obj);
+  int ret = store->delete_obj(rctx, obj);
 
   return ret;
 }
 
 static int remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_children)
 {
-  int ret;
   map<RGWObjCategory, RGWBucketStats> stats;
   std::vector<RGWObjEnt> objs;
   std::string prefix, delim, marker, ns;
@@ -727,7 +725,8 @@ static int remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_childr
   RGWBucketInfo info;
   bufferlist bl;
 
-  ret = store->get_bucket_stats(bucket, stats);
+  int ret = store->get_bucket_stats(bucket, stats);
+
   if (ret < 0)
     return ret;
 
@@ -750,7 +749,7 @@ static int remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_childr
     if (ret < 0)
       return ret;
 
-    while (objs.size() > 0) {
+    while (!objs.empty()) {
       std::vector<RGWObjEnt>::iterator it = objs.begin();
       for (it = objs.begin(); it != objs.end(); it++) {
         ret = remove_object(store, bucket, (*it).name);
@@ -945,7 +944,7 @@ int main(int argc, char **argv)
     }
   }
 
-  if (args.size() == 0) {
+  if (args.empty()) {
     return usage();
   }
   else {
@@ -1558,7 +1557,7 @@ next:
     if (rgw_read_user_buckets(store, user_id, buckets, false) >= 0) {
       map<string, RGWBucketEnt>& m = buckets.get_buckets();
 
-      if (m.size() > 0 && purge_data) {
+      if (!m.empty() && purge_data) {
         for (std::map<string, RGWBucketEnt>::iterator it = m.begin(); it != m.end(); it++) {
           ret = remove_bucket(store, ((*it).second).bucket, true);
 
@@ -1567,7 +1566,7 @@ next:
         }
       }
 
-      if (m.size() > 0 && !purge_data) {
+      if (!m.empty() && !purge_data) {
         cerr << "ERROR: specify --purge-data to remove a user with a non-empty bucket list" << std::endl;
         return 1;
       }
