@@ -62,7 +62,6 @@ protected:
   const char *if_match;
   const char *if_nomatch;
   off_t ofs;
-  uint64_t len;
   uint64_t total_len;
   off_t start;
   off_t end;
@@ -76,6 +75,7 @@ protected:
   bool get_data;
   bool partial_content;
   rgw_obj obj;
+  utime_t gc_invalidate_time;
 
   int init_common();
 public:
@@ -87,7 +87,6 @@ public:
     if_nomatch = NULL;
     start = 0;
     ofs = 0;
-    len = 0;
     total_len = 0;
     end = -1;
     mod_time = 0;
@@ -112,8 +111,10 @@ public:
                                   uint64_t *ptotal_len, bool read_data);
   int handle_user_manifest(const char *prefix);
 
+  int get_data_cb(bufferlist& bl, off_t ofs, off_t len);
+
   virtual int get_params() = 0;
-  virtual int send_response_data(bufferlist& bl) = 0;
+  virtual int send_response_data(bufferlist& bl, off_t ofs, off_t len) = 0;
 
   virtual const char *name() { return "get_obj"; }
 };
@@ -337,20 +338,12 @@ protected:
   map<string, bufferlist> attrs;
 
 public:
-  RGWPostObj() {}
+  RGWPostObj() : min_len(0), max_len(LLONG_MAX), ret(0), len(0), ofs(0),
+		 supplied_md5_b64(NULL), supplied_etag(NULL),
+		 data_pending(false) {}
 
   virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
     RGWOp::init(store, s, h);
-    min_len = 0;
-    max_len = LLONG_MAX;
-    ret = 0;
-    len = 0;
-    ofs = 0;
-    supplied_md5_b64 = NULL;
-    supplied_etag = NULL;
-    etag = "";
-    boundary = "";
-    data_pending = false;
     policy.set_ctx(s->cct);
   }
 
@@ -500,7 +493,7 @@ public:
   int verify_permission();
   void execute();
 
-  virtual int get_canned_policy(ACLOwner& owner, stringstream& ss) { return 0; }
+  virtual int get_policy_from_state(RGWRados *store, struct req_state *s, stringstream& ss) { return 0; }
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "put_acls"; }
@@ -536,12 +529,14 @@ protected:
   string etag;
   char *data;
   int len;
+  uint64_t min_part_size;
 
 public:
   RGWCompleteMultipart() {
     ret = 0;
     data = NULL;
     len = 0;
+    min_part_size = RGW_MIN_MULTIPART_SIZE;
   }
   virtual ~RGWCompleteMultipart() {
     free(data);
