@@ -3,6 +3,7 @@
 
 #include "include/rados/librados.hpp"
 #include "include/Context.h"
+#include "common/RefCountedObj.h"
 #include "rgw_common.h"
 #include "cls/rgw/cls_rgw_types.h"
 #include "rgw_log.h"
@@ -55,6 +56,12 @@ struct RGWUsageIter {
   RGWUsageIter() : index(0) {}
 };
 
+class RGWGetDataCB {
+public:
+  virtual int handle_data(bufferlist& bl, off_t bl_ofs, off_t bl_len) = 0;
+  virtual ~RGWGetDataCB() {}
+};
+
 class RGWAccessListFilter {
 public:
   virtual ~RGWAccessListFilter() {}
@@ -173,12 +180,11 @@ struct RGWObjState {
   bool has_data;
   bufferlist data;
   bool prefetch_data;
-  bool keep_tail;
 
   map<string, bufferlist> attrset;
   RGWObjState() : is_atomic(false), has_attrs(0), exists(false),
                   size(0), mtime(0), epoch(0), fake_tag(false), has_manifest(false),
-                  has_data(false), prefetch_data(false), keep_tail(false) {}
+                  has_data(false), prefetch_data(false) {}
 
   bool get_attr(string name, bufferlist& dest) {
     map<string, bufferlist>::iterator iter = attrset.find(name);
@@ -626,7 +632,24 @@ public:
 
   virtual void finish_get_obj(void **handle);
 
- /**
+  int iterate_obj(void *ctx, rgw_obj& obj,
+                  off_t ofs, off_t end,
+                  uint64_t max_chunk_size,
+                  int (*iterate_obj_cb)(rgw_obj&, off_t, off_t, off_t, bool, RGWObjState *, void *),
+                  void *arg);
+
+  int get_obj_iterate(void *ctx, void **handle, rgw_obj& obj,
+                      off_t ofs, off_t end,
+	              RGWGetDataCB *cb);
+
+  int get_obj_iterate_cb(void *ctx, RGWObjState *astate,
+                         rgw_obj& obj,
+                         off_t obj_ofs, off_t read_ofs, off_t len,
+                         bool is_head_obj, void *arg);
+
+  void get_obj_aio_completion_cb(librados::completion_t cb, void *arg);
+
+  /**
    * a simple object read without keeping state
    */
   virtual int read(void *ctx, rgw_obj& obj, off_t ofs, size_t size, bufferlist& bl);
@@ -695,15 +718,9 @@ public:
                             utime_t& ut, string& etag, string& content_type, bufferlist *acl_bl, RGWObjCategory category,
 			    list<string> *remove_objs);
   int complete_update_index_del(rgw_bucket& bucket, string& oid, string& tag, uint64_t epoch) {
-    if (bucket_is_system(bucket))
-      return 0;
-
     return cls_obj_complete_del(bucket, tag, epoch, oid);
   }
   int complete_update_index_cancel(rgw_bucket& bucket, string& oid, string& tag) {
-    if (bucket_is_system(bucket))
-      return 0;
-
     return cls_obj_complete_cancel(bucket, tag, oid);
   }
 
