@@ -348,7 +348,7 @@ EOF
 			echo
 		fi
 
-	        $SUDO $CEPH_BIN/ceph-authtool --create-keyring --gen-key --name=mon. $keyring_fn
+	        $SUDO $CEPH_BIN/ceph-authtool --create-keyring --gen-key --name=mon. $keyring_fn --cap mon 'allow *'
 	        $SUDO $CEPH_BIN/ceph-authtool --gen-key --name=client.admin --set-uid=0 \
 		    --cap mon 'allow *' \
 		    --cap osd 'allow *' \
@@ -380,6 +380,9 @@ EOF
 		    cmd="rm -rf dev/mon.$f"
 		    echo $cmd
 		    $cmd
+                    cmd="mkdir dev/mon.$f"
+                    echo $cmd
+                    $cmd
 		    cmd="$CEPH_BIN/ceph-mon --mkfs -c $conf -i $f --monmap=$monmap_fn"
 		    cmd="$cmd --keyring=$keyring_fn"
 		    echo $cmd
